/* Last edited on 2007-01-04 00:14:18 by stolfi */

#include <delaunay.h>
#include <delaunay_plot.h>

#include <quad.h>
#include <affirm.h>
#include <ps.h>

#include <ioprotos.h>
#include <stdio.h>
#include <math.h>
#include <string.h>
#include <stdlib.h>

#define NORMAL 1
#define NTOSS 9
#define VERBOSE 0
#define RBIG 10.0

void srandom(int);
int random(void);
double drandom(void);

site_struct *makesites(int nsites, bool_t normal);

int main(int argc, char **argv);

int main(int argc, char **argv)
  { int nsites = atoi(argv[1]);
    bool_t normal = (strcmp(argv[2], "normal") == 0);
    bool_t eps = (strcmp(argv[3], "eps") == 0);
    quad_arc_t e;
    site_struct *st = makesites(nsites, normal);
    e = delaunay_build (st, nsites);
    printf("Delaunay returned %x:%d \n", (e & 0xfffffffcu), (e&3));
    plot_delaunay(e, st, nsites, "delrandom", eps);
    return(0);
  }

site_struct *makesites(int nsites, bool_t normal)
  { int i, j;
    double s, t;
    site_struct *st = notnull(malloc(nsites*sizeof(site_struct)), "no mem");
    
    srandom(4615);

    for (i=0; i < nsites; i++) 
      { if (normal) 
          { /* central limit approx to normal distribuition: */
            s = t = 0.0;
            for (j=0; j<NTOSS; j++)
              { s += 2 * (drandom() - 0.5);
                t += 2 * (drandom() - 0.5);
              }
            st[i].x = s/sqrt(NTOSS);
            st[i].y = t/sqrt(NTOSS);
          }
        else 
          { /* uniform distribution: */
            st[i].x = 2 * (drandom() - 0.5);
            st[i].y = 2 * (drandom() - 0.5);
          }
        if (VERBOSE) 
          { fprintf(stderr, "%3d (%12.8f, %12.8f)\n", i, st[i].x, st[i].y); }
      }
    return st;
  }
