/* See r6x6.h. */
/* Last edited on 2007-01-03 22:42:55 by stolfi */

#include <r6x6.h>
#include <r6.h>
#include <math.h>
#include <stdio.h>
#include <ioprotos.h>
#include <rmxn.h>

#define N 6

void r6x6_zero(r6x6_t *R)
  { int i, j;
    for (i = 0; i < N; i++)
      for (j = 0; j < N; j++)
        { R->c[i][j] = 0.0; }
  }

void r6x6_ident(r6x6_t *R)
  { int i, j;
    for (i = 0; i < N; i++)
      for (j = 0; j < N; j++)
        { R->c[i][j] = (i == j ? 1.0 : 0.0); }
  }

void r6x6_map_row (r6_t *x, r6x6_t *A, r6_t *R)
  { int j, k;
    for (j = 0; j < N; j++)
      { double s = 0.0;
        for (k = 0; k < N; k++) s += x->c[k] * A->c[k][j];
        R->c[j] = s;
      }
  }

void r6x6_map_col (r6x6_t *A, r6_t *x, r6_t *R)
  { int i, k;
    for (i = 0; i < N; i++)
      { double s = 0.0;
        for (k = 0; k < N; k++) s += A->c[i][k] * x->c[k];
        R->c[i] = s;
      }
  }

void r6x6_mul (r6x6_t *A, r6x6_t *B, r6x6_t *R)
  { int i, j, k;
    for (i = 0; i < N; i++)
      for (j = 0; j < N; j++)
        { double s = 0.0;
          for (k = 0; k < N; k++)  s += A->c[i][k]*B->c[k][j];
          R->c[i][j] = s;
        }
  }

double r6x6_det (r6x6_t *A)
  { return rmxn_det(N, &(A->c[0][0])); }

void r6x6_inv (r6x6_t *A, r6x6_t *R)
  { rmxn_inv(N, &(A->c[0][0]), &(R->c[0][0])); }

void r6x6_print (FILE *f, r6x6_t *A)
  { r6x6_gen_print(f, A, NULL, NULL, NULL, NULL, NULL, NULL, NULL); }

void r6x6_gen_print 
  ( FILE *f, r6x6_t *A,
    char *fmt, 
    char *olp, char *osep, char *orp,
    char *ilp, char *isep, char *irp
  )
  { rmxn_gen_print(f, N, N, &(A->c[0][0]), fmt, olp, osep, orp, ilp, isep, irp); }

