/* See fn2_zf_grid.h */
/* Last edited on 2007-01-04 00:15:54 by stolfi */

#include <fn2_functions.h>
#include <fn2_zf_grid.h>
#include <fn2_zf_flt.h>

#include <aa.h>
#include <ia.h>
#include <flt.h>
#include <ps.h>

#include <math.h>
#include <stdio.h>
#include <malloc.h>

/*** PROTOTYPES FOR INTERNAL PROCEDURES ***/

char *fn2_zf_grid_format_parms(
    Interval xd,
    Interval yd,
    int n
  );
  /* Formats the arguments into a sctring */
  
void fn2_zf_grid_report_stats(
    FILE *psfile,
    int epsformat, 
    char *arith,    /* "IA" or "AA" */
    int nevals, 
    int ncells
  );
  /* Prints evaluation report, and adds caption lines if not $epsformat$ */
  
void fn2_zf_grid_plot_aa_zeros(FILE *psfile,
    int epsformat,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n
  );

void fn2_zf_grid_plot_ia_zeros(
    FILE *psfile,
    int epsformat,
    eval_ia_t *eval_ia,
    Interval xd,
    Interval yd,
    int n
  );

/*** IMPLEMENTATIONS ***/

void fn2_zf_grid_plots(
    char *fileprefix,
    int epsformat,
    char *title,
    eval_fp_t *eval_fp,
    eval_ia_t *eval_ia,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n,
    int m
  )
  { char *parmstr = fn2_zf_grid_format_parms(xd, yd, n);
    FILE *psfile;
    Float wx = xd.hi - xd.lo;
    Float wy = yd.hi - yd.lo;
    Float wm = FMAX(wx, wy);
    double scale = 6.0 * 72.0 / wm;
    double xc = 4.25 * 72.0;
    double hmin = xc - scale * wx / 2.0;
    double hmax = xc + scale * wx / 2.0;
    double yc = 6.50 *72.0;
    double vmin = yc - scale * wy / 2.0;
    double vmax = yc + scale * wy / 2.0;

    /*** Ordinary interval arithmetic plot ***/
    
    if (epsformat)
      { psfile = open_write(txtcat(fileprefix, "-ia.eps"), TRUE);
        ps_begin_figure(psfile, 0.0, hmax-hmin+8.0, 0.0, vmax-vmin+8.0);
        ps_set_window(psfile,
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  4.0, hmax-hmin+4.0, 4.0, vmax-vmin+4.0,
	  n, n
	);
        ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
      }
    else
      { psfile = open_write(txtcat(fileprefix, ".ps"), TRUE);
        ps_begin_document(psfile, "letter");
        ps_begin_page(psfile, "1");
	ps_set_window(psfile,
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  hmin,  hmax,  vmin,  vmax,
	  n, n
	);
        ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
        ps_add_caption(psfile, title);
        ps_add_caption(psfile, "");
        ps_add_caption(psfile, parmstr);
        ps_add_caption(psfile, "");
        ps_add_caption(psfile, "Ordinary interval arithmetic");
      }

    fn2_zf_grid_plot_ia_zeros(psfile, epsformat, eval_ia, xd, yd, n);
    
    ps_set_pen(psfile, 0.5,0.0,0.0, 0.25, 0.0, 0.0);
    fn2_zf_flt_plot(psfile, eval_fp, xd, yd, m);
    
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
    ps_draw_grid_lines(psfile);
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.25, 0.0, 0.0);
    ps_draw_frame(psfile);

    if (epsformat)
      { ps_end_figure(psfile); }
    else
      { ps_end_page(psfile); }
      
    /*** Affine arithmetic plot ***/

    if (epsformat)
      { psfile = open_write(txtcat(fileprefix, "-aa.eps"), TRUE);
        ps_begin_figure(psfile, 0.0, hmax-hmin+8.0, 0.0, vmax-vmin+8.0);
        ps_set_window(psfile,
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  4.0, hmax-hmin+4.0, 4.0, vmax-vmin+4.0,
	  n, n
	);
        ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
      }
    else
      { ps_begin_page(psfile, "2");
	ps_set_window(psfile,
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  hmin,  hmax,  vmin,  vmax,
	  n, n
	);
        ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
	ps_add_caption(psfile, title);
	ps_add_caption(psfile, "");
	ps_add_caption(psfile, parmstr);
	ps_add_caption(psfile, "");
	ps_add_caption(psfile, "Affine arithmetic");
      }

    fn2_zf_grid_plot_aa_zeros(psfile, epsformat, eval_aa, xd, yd, n);
    
    ps_set_pen(psfile, 0.5,0.0,0.0, 0.25, 0.0, 0.0);
    fn2_zf_flt_plot(psfile, eval_fp, xd, yd, m);
    
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
    ps_draw_grid_lines(psfile);
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.25, 0.0, 0.0);
    ps_draw_frame(psfile);

    if (epsformat) 
      { ps_end_figure(psfile); }
    else
      { ps_end_page(psfile);
        ps_end_document(psfile);
        fclose(psfile);
      }
  }

char *fn2_zf_grid_format_parms(
    Interval xd,
    Interval yd,
    int n
  )
  { char *s = (char *) malloc(100);
    sprintf(s, 
      "x in [%f _ %f]  y in [%f _ %f]  %d intervals",
      xd.lo, xd.hi, yd.lo, yd.hi, n
    );
    return(s);
  }

void fn2_zf_grid_plot_aa_zeros(
    FILE *psfile,
    int epsformat,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n
  )
  { Interval xv, yv, zv;
    int xi, yi;
    AAP xf, yf, zf;
    double gray = 0.75;
    int ncells = 0;
    char sevals[80];

    fprintf(stderr, "\nenter fn2_zf_grid_plot_aa_zeros...\n");

    ps_comment(psfile, "Plot of AA-arithmetic zeros");

    for (yi=0; yi<n; yi++)
      { for (xi=0; xi<n; xi++)
          { MemP frame = aa_top();

            ROUND_DOWN;
            xv.lo = xd.lo + ((xd.hi - xd.lo)*xi)/n;
            yv.lo = yd.lo + ((yd.hi - yd.lo)*yi)/n;

            ROUND_UP;
            xv.hi = xd.lo + ((xd.hi - xd.lo)*(xi+1))/n;
            yv.hi = yd.lo + ((yd.hi - yd.lo)*(yi+1))/n;

            xf = aa_from_interval(xv);
            yf = aa_from_interval(yv);

            zf = eval_aa(xf, yf);
            zv = aa_range(zf);

            ROUND_NEAR;
            if ((zv.lo <= Zero) && (zv.hi >= Zero))
              { ps_fill_grid_cell(psfile, xi, yi, gray, gray, gray);
                ncells++;
              }

            aa_flush(frame);
          }
      }

    fprintf(stderr, " done\n");
    fprintf(stderr, "%d unresolved cells\n", ncells);
    if (! epsformat)
      { sprintf(sevals, "%d unresolved cells", ncells);
        ps_add_caption(psfile, sevals);
      }
    fprintf(stderr, "exit fn2_zf_grid_plot_aa_zeros.\n");
  }

void fn2_zf_grid_plot_ia_zeros(
    FILE *psfile,
    int epsformat,
    eval_ia_t *eval_ia,
    Interval xd,
    Interval yd,
    int n
  )
  { Interval xv, yv, zv;
    int xi, yi;
    double gray = 0.75;
    int ncells = 0;
    char sevals[80];

    fprintf(stderr, "\nenter fn2_zf_grid_plot_ia_zeros...\n");

    ps_comment(psfile, "Plot of interval-arithmetic zeros");

    for (yi=0; yi<n; yi++)
      { for (xi=0; xi<n; xi++)
          { ROUND_DOWN;
            xv.lo = xd.lo + ((xd.hi - xd.lo)*xi)/n;
            yv.lo = yd.lo + ((yd.hi - yd.lo)*yi)/n;

            ROUND_UP;
            xv.hi = xd.lo + ((xd.hi - xd.lo)*(xi+1))/n;
            yv.hi = yd.lo + ((yd.hi - yd.lo)*(yi+1))/n;

            zv = eval_ia(xv, yv);

            ROUND_NEAR;
            if ((zv.lo <= Zero) && (zv.hi >= Zero))
              { ps_fill_grid_cell(psfile, xi, yi, gray, gray, gray);
                ncells++;
              }
          }
      }

    fprintf(stderr, " done\n");
    fprintf(stderr, "%d unresolved cells\n", ncells);
    if (! epsformat)
      { sprintf(sevals, "%d unresolved cells", ncells);
        ps_add_caption(psfile, sevals);
      }
    fprintf(stderr, "exit fn2_zf_grid_plot_ia_zeros.\n");
  }

