/* See fn2_zf_quad.h */
/* Last edited on 2007-01-04 03:52:55 by stolfi */

#include "fn2_zf_quad.h"
#include "fn2_zf_flt.h"

#include <aa.h>
#include <ia.h>
#include <flt.h>
#include <ps.h>
#include <bool.h>
#include <jsfile.h>
#include <jsstring.h>

#include <math.h>
#include <stdio.h>
#include <malloc.h>

/*** PROTOTYPES FOR INTERNAL ROUTINES ***/

void fn2_zf_quad_plot_aa_zeros(
    FILE *psfile,
    int epsformat,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n
  );
  /* 
    Plots roots of {f(xf,yf) = 0}, using {eval_aa} to 
    evaluate it with affine arithmetic.
    The input intervals are {n}{n} cells in {xd}{yd}. */

char *fn2_zf_quad_format_parms(
    Interval xd,
    Interval yd,
    int n
  );
  /* Formats the arguments into a string */
  
void fn2_zf_quad_report_stats(
    FILE *psfile,
    int epsformat, 
    char *arith,    /* "IA" or "AA" */
    int nevals, 
    int ncells
  );
  /* Prints evaluation report, and adds caption lines if not $epsformat$ */
  
void fn2_zf_quad_aa_zeros_aux(
    FILE *psfile,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n,
    int *nevals,
    int *ncells,
    int ixlo, int mx,
    int iylo, int my
  );
  /* Called by fn2_zf_quad_plot_aa_zeros */

void fn2_zf_quad_plot_ia_zeros(
    FILE *psfile,
    int epsformat,
    eval_ia_t *eval_ia,
    Interval xd,
    Interval yd,
    int n
  );
  /* Plots zeros of fv(xv,yv) = 0, using ordinary interval arithmetic.
    The input inervals are nxn cells in xd x yd. */

void fn2_zf_quad_ia_zeros_aux(
    FILE *psfile,
    eval_ia_t *eval_ia,
    Interval xd,
    Interval yd,
    int n,
    int *nevals,
    int *ncells,
    int ixlo, int mx,
    int iylo, int my
  );
  /* Called by fn2_zf_quad_plot_ia_zeros */

/*** IMPLEMENTATIONS ***/

void fn2_zf_quad_plots(
    char *fileprefix,
    int epsformat,
    char *title,
    eval_fp_t *eval_fp,
    eval_ia_t *eval_ia,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n,
    int m
  )
  { char *parmstr = fn2_zf_quad_format_parms(xd, yd, n);
    FILE *psfile;
    Float wx = xd.hi - xd.lo;
    Float wy = yd.hi - yd.lo;
    Float wm = FMAX(wx, wy);
    double scale = 6.0 * 72.0 / wm;
    double xc = 4.25 * 72.0;
    double hmin = xc - scale * wx / 2.0;
    double hmax = xc + scale * wx / 2.0;
    double yc = 6.50 *72.0;
    double vmin = yc - scale * wy / 2.0;
    double vmax = yc + scale * wy / 2.0;

    /*** Ordinary interval arithmetic plot ***/
    
    if (epsformat)
      { psfile = open_write(txtcat(fileprefix, "-ia.eps"), TRUE);
        ps_begin_figure(psfile, 0.0, hmax-hmin, 0.0, vmax-vmin);
        ps_set_window(psfile, 
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  0.0, hmax-hmin, 0.0, vmax-vmin,
	  n, n
	);
      }
    else
      { psfile = open_write(txtcat(fileprefix, ".ps"), TRUE);
        ps_begin_document(psfile, "letter");
        ps_begin_page(psfile, "1");
	ps_set_window(psfile,
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  hmin, hmax, vmin, vmax,
	  n, n
	);
        ps_add_caption(psfile, title);
        ps_add_caption(psfile, "");
        ps_add_caption(psfile, parmstr);
        ps_add_caption(psfile, "");
        ps_add_caption(psfile, "Ordinary interval arithmetic");
      }

    ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
    fn2_zf_quad_plot_ia_zeros(psfile, epsformat, eval_ia, xd, yd, n);
    ps_set_pen(psfile, 0.5,0.0,0.0, 0.10, 0.0, 0.0);
    fn2_zf_flt_plot(psfile, eval_fp, xd, yd, m);
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
    ps_draw_frame(psfile);

    if (epsformat)
      { ps_end_figure(psfile); }
    else
      { ps_end_page(psfile); }
      
    /*** Affine arithmetic plot ***/

    if (epsformat)
      { psfile = open_write(txtcat(fileprefix, "-aa.eps"), TRUE);
	ps_begin_figure(psfile, 0.0, hmax-hmin, 0.0, vmax-vmin);
        ps_set_window(psfile,
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  0.0, hmax-hmin, 0.0, vmax-vmin,
	  n, n
	);
        ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
      }
    else
      { ps_begin_page(psfile, "2");
	ps_set_window(psfile,
	  xd.lo, xd.hi, yd.lo, yd.hi,
	  hmin,  hmax,  vmin,  vmax,
	  n, n
	);
	ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
        ps_add_caption(psfile, title);
	ps_add_caption(psfile, "");
	ps_add_caption(psfile, parmstr);
	ps_add_caption(psfile, "");
	ps_add_caption(psfile, "Affine arithmetic");
      }

    fn2_zf_quad_plot_aa_zeros(psfile, epsformat, eval_aa, xd, yd, n);
    ps_set_pen(psfile, 0.5,0.0,0.0, 0.10, 0.0, 0.0);
    fn2_zf_flt_plot(psfile, eval_fp, xd, yd, m);
    ps_set_pen(psfile, 0.0,0.0,0.0, 0.10, 0.0, 0.0);
    ps_draw_frame(psfile);

    if (epsformat) 
      { ps_end_figure(psfile); }
    else
      { ps_end_page(psfile);
        ps_end_document(psfile);
        fclose(psfile);
      }
  }

char *fn2_zf_quad_format_parms(
    Interval xd,
    Interval yd,
    int n
  )
  { char *s = (char *) malloc(100);
    sprintf(s, 
      "x in [%f _ %f]  y in [%f _ %f]  %d intervals",
      xd.lo, xd.hi, yd.lo, yd.hi, n
    );
    return(s);
  }

void fn2_zf_quad_plot_aa_zeros(
    FILE *psfile,
    int epsformat,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n
  )
  { int nevals = 0;
    int ncells = 0;

    fprintf(stderr, "\nenter fn2_zf_quad_plot_aa_zeros...\n");

    ps_comment(psfile, "Plot of AA-arithmetic zeros");

    fn2_zf_quad_aa_zeros_aux(
      psfile, eval_aa, xd, yd, n, &nevals, &ncells,
      0, n, 0, n
    );

    fprintf(stderr, " done\n");
    
    fn2_zf_quad_report_stats(psfile, epsformat, "AA", nevals, ncells);

    fprintf(stderr, "exit fn2_zf_quad_plot_aa_zeros.\n");
  }

void fn2_zf_quad_aa_zeros_aux(
    FILE *psfile,
    eval_aa_t *eval_aa,
    Interval xd,
    Interval yd,
    int n,
    int *nevals,
    int *ncells,
    int ixlo, int mx,
    int iylo, int my
  )
  { Interval xv, yv, zv;
    AAP xf, yf, zf;
    double gray = 0.75;

    MemP frame = aa_top();

    fprintf(stderr, "(");

    ROUND_DOWN;
    xv.lo = xd.lo + ((xd.hi - xd.lo)*ixlo)/n;
    yv.lo = yd.lo + ((yd.hi - yd.lo)*iylo)/n;

    ROUND_UP;
    xv.hi = xd.lo + ((xd.hi - xd.lo)*(ixlo+mx))/n;
    yv.hi = yd.lo + ((yd.hi - yd.lo)*(iylo+my))/n;

    xf = aa_from_interval(xv);
    yf = aa_from_interval(yv);

    zf = eval_aa(xf, yf);
    zv = aa_range(zf);
    (*nevals)++;

    aa_flush(frame);

    ROUND_NEAR;
    if ((zv.lo > Zero) || (zv.hi < Zero))
      { ps_draw_rectangle(psfile, xv.lo, xv.hi, yv.lo, yv.hi); }
    else if ((mx == 1) && (my == 1))
      { (*ncells)++;
        ps_fill_and_draw_rectangle(psfile, 
          xv.lo, xv.hi, yv.lo, yv.hi, 
          gray, gray, gray
        );
      }
    else if (mx >= my)
      { int mx2 = mx / 2;
        fn2_zf_quad_aa_zeros_aux(
          psfile, eval_aa, xd, yd, n, nevals, ncells,
          ixlo, mx2, iylo, my
        );
        fn2_zf_quad_aa_zeros_aux(
          psfile, eval_aa, xd, yd, n, nevals, ncells,
          ixlo + mx2, mx - mx2, iylo, my
        );
      }
    else /* if (my > mx) */
      { int my2 = my / 2;
        fn2_zf_quad_aa_zeros_aux(
          psfile, eval_aa, xd, yd, n, nevals, ncells,
          ixlo, mx, iylo, my2
        );
        fn2_zf_quad_aa_zeros_aux(
          psfile, eval_aa, xd, yd, n, nevals, ncells,
          ixlo, mx, iylo + my2, my - my2
        );
      }

    fprintf(stderr, ")");
  }

void fn2_zf_quad_plot_ia_zeros(
    FILE *psfile,
    int epsformat,
    eval_ia_t *eval_ia,
    Interval xd,
    Interval yd,
    int n
  )
  { int nevals = 0;
    int ncells = 0;

    fprintf(stderr, "\nenter fn2_zf_quad_plot_ia_zeros...\n");

    ps_comment(psfile, "Plot of interval-arithmetic zeros");

    fn2_zf_quad_ia_zeros_aux(
      psfile, eval_ia, xd, yd, n, &nevals, &ncells,
      0, n, 0, n
    );

    fprintf(stderr, "\n");
    
    fn2_zf_quad_report_stats(psfile, epsformat, "IA", nevals, ncells);

    fprintf(stderr, "exit fn2_zf_quad_plot_ia_zeros\n");
  }

void fn2_zf_quad_report_stats(
    FILE *psfile, 
    int epsformat, 
    char *arith, 
    int nevals, 
    int ncells
  )
  { char sevals[80];

    fprintf(stderr, "%d %s function evaluations\n", nevals, arith);
    if (! epsformat)
      { sprintf(sevals, "%d %s function evaluations", nevals, arith);
        ps_add_caption(psfile, sevals);
      }

    fprintf(stderr, "%d cells retained\n", ncells);
    if (! epsformat)
      { sprintf(sevals, "%d cells retained", ncells);
        ps_add_caption(psfile, sevals);
      }
  }

void fn2_zf_quad_ia_zeros_aux(
    FILE *psfile,
    eval_ia_t *eval_ia,
    Interval xd,
    Interval yd,
    int n,
    int *nevals,
    int *ncells,
    int ixlo, int mx,
    int iylo, int my
  )
  { Interval xv, yv, zv;
    double gray = 0.75;

    fprintf(stderr, "(");

    ROUND_DOWN;
    xv.lo = xd.lo + ((xd.hi - xd.lo)*ixlo)/n;
    yv.lo = yd.lo + ((yd.hi - yd.lo)*iylo)/n;

    ROUND_UP;
    xv.hi = xd.lo + ((xd.hi - xd.lo)*(ixlo+mx))/n;
    yv.hi = yd.lo + ((yd.hi - yd.lo)*(iylo+my))/n;

    zv = eval_ia(xv, yv);
    (*nevals)++;

    ROUND_NEAR;
    if ((zv.lo > Zero) || (zv.hi < Zero))
      { ps_draw_rectangle(psfile, xv.lo, xv.hi, yv.lo, yv.hi); }
    else if ((mx == 1) && (my == 1))
      { ps_fill_and_draw_rectangle(
          psfile, xv.lo, xv.hi, yv.lo, yv.hi, 
          gray, gray, gray
        );
        (*ncells)++;
      }
    else if (mx >= my)
      { int mx2 = mx / 2;
        fn2_zf_quad_ia_zeros_aux(
          psfile, eval_ia, xd, yd, n, nevals, ncells,
          ixlo, mx2, iylo, my
        );
        fn2_zf_quad_ia_zeros_aux(
          psfile, eval_ia, xd, yd, n, nevals, ncells,
          ixlo + mx2, mx - mx2, iylo, my
        );
      }
    else /* if (my > mx) */
      { int my2 = my / 2;
        fn2_zf_quad_ia_zeros_aux(
          psfile, eval_ia, xd, yd, n, nevals, ncells,
          ixlo, mx, iylo, my2
        );
        fn2_zf_quad_ia_zeros_aux(
          psfile, eval_ia, xd, yd, n, nevals, ncells,
          ixlo, mx, iylo + my2, my - my2
        );
      }

    fprintf(stderr, ")");
  }

