# Generic Makefile for a single test program
# Last edited on 2007-04-16 21:08:14 by stolfi

######################################################################
# To be included by the top-level Makefile in the program's directory.
# The caller must define ${PROG} and possibly also ${JSLIBS}

ifneq "/${PROG}" "/"

SHELL := /bin/sh

# Where to install the program:
INSTPUB := ${HOME}/PUB
INSTINC := ${INSTPUB}/include
INSTLIB := ${INSTPUB}/${PLATFORM}/lib
INSTBIN := ${INSTPUB}/${PLATFORM}/bin
INSTMAN := ${INSTPUB}/${PLATFORM}/man
INSTCAT := ${INSTPUB}/${PLATFORM}/cat

# Location of the GL libraries
GLLIB := /usr/lib

# Location of X11 libraries (including ICE):
X11INC := /usr/include/X11 
X11LIB := /usr/lib

# Location of Stolfi's headers and libraries:
JSPUB := ${HOME}/PUB
JSLIB := ${JSPUB}/${PLATFORM}/lib
JSINC := ${JSPUB}/include

# Get list of source files (excluding ${PROG}.c):
HFILES := ${filter-out Makefile, ${shell shopt -s nullglob; ls *.h Makefile}}
CFILES := ${filter-out Makefile ${PROG}.c, ${shell shopt -s nullglob; ls *.c Makefile}}

# Derived object files (excluding ${PROG}.o):
HOFILES := ${subst .h,.ho,${HFILES}}
OFILES := ${subst .c,.o,${CFILES}}

#---------------------------------------------------------------------
# Phony make targets.
# The client must define the "all" and "clean" targets!

.PHONY: \
  default-action \
  all \
  clean \
  depend \
  build build-prog

#---------------------------------------------------------------------
# Default target: call the client's "all" target.
default-action: all

#---------------------------------------------------------------------
# "make depend" recreates the source dependency file ${DEPFILE}.

# File with dependencies between sources:
DEPFILE := Deps.make
# This file must be present to run "make";
# create an empty one if necessary.

depend: 
	/bin/rm -f ${DEPFILE}
	extract-ho-deps ${IFLAGS} -I/usr/include \
            ${HFILES} ${CFILES} ${PROG}.c \
          | egrep -v ': /usr/include' \
          > ${DEPFILE}

# Include specific dependencies extracted by "make depend"
include ${DEPFILE}

#---------------------------------------------------------------------
# "make build" assumes that the dependencies in ${DEPFILE}
# are up-to-date.

# Directories to search for #include files:
IFLAGS := -I. -I${JSINC}

build: build-prog
  
build-prog: ${PROG}
  
# Get OpenGL libraries is requested:
ifeq "/${USEGL}" "/YES" 
  GL_LIBRARIES := \
    ${JSLIB}/libglut.so \
    ${GLLIB}/libGLU.so \
    ${GLLIB}/libGL.so
else
  GL_LIBRARIES :=
endif

# Get X11 libraries if requested:
ifeq "/${USEX11}" "/YES" 
  X11_LIBRARIES := \
    ${X11LIB}/libX11.so \
    ${X11LIB}/libXext.so \
    ${X11LIB}/libXmu.so \
    ${X11LIB}/libXt.so \
    ${X11LIB}/libXi.so \
    ${X11LIB}/libSM.so \
    ${X11LIB}/libICE.so
else
  X11_LIBRARIES := 
endif
  
LIBRARIES := \
  ${addprefix ${JSLIB}/, ${JSLIBS}} \
  ${addprefix ${PBMLIB}/, ${PBMLIBS}} \
  ${GL_LIBRARIES} \
  ${X11_LIBRARIES} \
  ${OTHERLIBS}

CC := /usr/bin/gcc

CFLAGS := \
  -ggdb \
  -Wall -Wpointer-arith -Wmissing-prototypes \
  -fpcc-struct-return -ffloat-store -frounding-math
  
LDFLAGS := \
  -ggdb

%.o: %.c
	${CC} -c ${CFLAGS} ${EXTRACFLAGS} ${IFLAGS} $*.c
  
%.ho: %.h
	${CC} -o $*.ho -c ${CFLAGS} ${IFLAGS} -x c $*.h \
          || /bin/rm -f $*.ho
  
${PROG}: ${PROG}.o ${OFILES} ${LIBRARIES} 
	-rm -f ${PROG}
	${CC} -o ${PROG} ${LDFLAGS} ${PROG}.o \
          ${OFILES} \
          ${LIBRARIES} -lm -lrt
        
endif
# End of ${PROG} section.
######################################################################

